"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getScreenshotPath = exports.addFrameSnapshots = exports.addDiffToManifest = exports.getChanges = exports.getProjectDiffs = exports.getDiffFrame = exports.getProjectScreenshots = exports.getScreenshotDiffs = exports.getFrameScreenshots = exports.getProjectFrames = exports.getProjectSnapshot = exports.sizeEquals = exports.screenshotEquals = exports.SCREENSHOTS_DIR = exports.DATA_DIR = exports.DIFF_DIR = exports.PC_HIDDEN_DIR = exports.DIFF_BOUNDARY = exports.MANIFEST_FILE_NAME = exports.MANIFEST_VERSION = void 0;
exports.MANIFEST_VERSION = 7;
const immer_1 = require("immer");
const path = require("path");
exports.MANIFEST_FILE_NAME = "manifest.json";
exports.DIFF_BOUNDARY = "~";
exports.PC_HIDDEN_DIR = ".paperclip";
exports.DIFF_DIR = exports.PC_HIDDEN_DIR + "/diff";
exports.DATA_DIR = exports.DIFF_DIR + "/data";
exports.SCREENSHOTS_DIR = exports.DATA_DIR + "/screenshots";
const screenshotEquals = (a, b) => a.browser === b.browser && (0, exports.sizeEquals)(a.size, b.size);
exports.screenshotEquals = screenshotEquals;
const sizeEquals = (a, b) => a.width === b.width && a.height === b.height;
exports.sizeEquals = sizeEquals;
const getProjectSnapshot = (version, manifest) => manifest.projectSnapshots.find(snapshot => snapshot.version === version);
exports.getProjectSnapshot = getProjectSnapshot;
const getProjectFrames = (project, manifest) => manifest.frameSnapshots.filter(snapshot => project.frameIds.includes(snapshot.id));
exports.getProjectFrames = getProjectFrames;
const getFrameScreenshots = (frame, manifest) => manifest.screenshots.filter(screenshot => frame.screenshotIds.includes(screenshot.id));
exports.getFrameScreenshots = getFrameScreenshots;
const getScreenshotDiffs = (screenshot, manifest) => manifest.diffs.filter(diff => screenshot.diffIds.includes(diff.id));
exports.getScreenshotDiffs = getScreenshotDiffs;
const getProjectScreenshots = (version, manifest) => {
    const project = (0, exports.getProjectSnapshot)(version, manifest);
    const screenshots = [];
    const frames = (0, exports.getProjectFrames)(project, manifest);
    for (const frame of frames) {
        screenshots.push(...(0, exports.getFrameScreenshots)(frame, manifest));
    }
    return screenshots;
};
exports.getProjectScreenshots = getProjectScreenshots;
const getDiffScreenshot = (diff, manifest) => {
    return manifest.screenshots.find(screenshot => screenshot.diffIds.includes(diff.id));
};
const getDiffFrame = (diff, manifest) => {
    const diffScreenshot = getDiffScreenshot(diff, manifest);
    return manifest.frameSnapshots.find(frame => frame.screenshotIds.includes(diffScreenshot.id));
};
exports.getDiffFrame = getDiffFrame;
const getProjectDiffs = (version, manifest) => {
    const screenshots = (0, exports.getProjectScreenshots)(version, manifest);
    const diffs = [];
    for (const screenshot of screenshots) {
        diffs.push(...(0, exports.getScreenshotDiffs)(screenshot, manifest));
    }
    return diffs;
};
exports.getProjectDiffs = getProjectDiffs;
const getChanges = (version1, version2, manifest) => {
    const otherScreenshotIds = (0, exports.getProjectScreenshots)(version2, manifest).map(v => v.id);
    const v1Diffs = (0, exports.getProjectDiffs)(version1, manifest);
    return v1Diffs.filter(diff => {
        return otherScreenshotIds.includes(diff.otherScreenshotId);
    });
};
exports.getChanges = getChanges;
const addDiffToManifest = (screenshotId, screenshotDiff, manifest) => {
    manifest = (0, immer_1.produce)(manifest, newManifest => {
        newManifest.diffs = insert("id", [screenshotDiff], manifest.diffs);
        for (const screenshot of newManifest.screenshots) {
            if (screenshot.id === screenshotId) {
                if (!screenshot.diffIds.includes(screenshotDiff.id)) {
                    screenshot.diffIds.push(screenshotDiff.id);
                }
                break;
            }
        }
    });
    return manifest;
};
exports.addDiffToManifest = addDiffToManifest;
const addFrameSnapshots = (projectSnapshot, frames, screenshots, gitDir, version, tagName, manifest) => __awaiter(void 0, void 0, void 0, function* () {
    manifest = (0, immer_1.produce)(manifest, newManifest => {
        newManifest.commitAliases[tagName] = version;
        newManifest.projectSnapshots = insert("version", [projectSnapshot], newManifest.projectSnapshots);
        newManifest.frameSnapshots = insert("id", frames, newManifest.frameSnapshots);
        newManifest.screenshots = insert("id", screenshots, newManifest.screenshots);
    });
    return manifest;
});
exports.addFrameSnapshots = addFrameSnapshots;
const insert = (key, newItems, collection) => [
    ...collection.filter(item => !newItems.some(aitem => aitem[key] === item[key])),
    ...newItems
];
const getScreenshotPath = (gitDir, id) => path.join(gitDir, exports.SCREENSHOTS_DIR, id + ".png");
exports.getScreenshotPath = getScreenshotPath;
